Imports System.Runtime.Serialization
Imports System.Threading

Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core

Namespace DomainModel

  ''' <summary>
  ''' An example of how to use the "push" feature for intermediate progress notifications.
  ''' <para>
  ''' This example is a very simple example of how to periodically broadcast information 
  ''' to client subscribers.  The data sent to subscribers is a non-primitive type, and must
  ''' therefore be both serializable and a known type.
  ''' </para>
  ''' </summary>
  Public Class LongRunningProcedure

	Private Shared __serviceKey As Guid
	Private Shared __notificationManager As INotificationManager
	Private Shared __entityManager As EntityManager

	''' <summary>
	''' Method "subscribed" to from a client in the RegisterCallback() method.  This method will start on a new
	''' thread upon the first client request, and run as long as there are subscribers or until the method exits.
	''' </summary>
	''' <param name="serviceKey">Uniquely identifies your "service".  Use this when calling the INotificationManager.</param>
	''' <param name="notificationManager">The INotificationManager you will use to request information about subscribers and send them data.</param>
	''' <param name="entityManager">A server-side EntityManager.</param>
	<AllowRpc>
	Public Shared Sub MultiStepService(ByVal serviceKey As Guid, ByVal notificationManager As INotificationManager, ByVal entityManager As EntityManager)

	  __serviceKey = serviceKey
	  __notificationManager = notificationManager
	  __entityManager = entityManager

	  TraceFns.WriteLine("LongRunningProcedure sample started")

	  ' Note this procedure starts only upon first subscription.  Subsequent subscribers will be notified
	  ' of current progress, but each doesn't start its own long running procedure.  You could do this by
	  ' periodically calling GetSubscribers, starting the procedure they request, and then tracking all
	  ' this.

	  Step1()
	  Step2()
	  Step3()
	  Step4()

	  TraceFns.WriteLine("LongRunningProcedure sample exiting")
	End Sub

	Private Shared Sub Step1()
	  ' Assume some logic is performed that represents 20% of the total workload.
	  Thread.Sleep(1000 * 5)
	  __notificationManager.Send(__serviceKey, New LongRunningInfo("The LongRunningProcedure is 20% done.", 20))
	End Sub

	Private Shared Sub Step2()
	  ' Assume some logic is performed that represents 50% of the total workload.
	  Thread.Sleep(1000 * 5)
	  __notificationManager.Send(__serviceKey, New LongRunningInfo("The LongRunningProcedure is 50% done.", 50))
	End Sub
	Private Shared Sub Step3()
	  ' Assume some logic is performed that represents 90% of the total workload.
	  Thread.Sleep(1000 * 5)
	  __notificationManager.Send(__serviceKey, New LongRunningInfo("The LongRunningProcedure is 90% done.", 90))
	End Sub

	Private Shared Sub Step4()
	  ' Assume some logic is performed that represents 100% of the total workload.
	  Thread.Sleep(1000 * 5)
	  __notificationManager.Send(__serviceKey, New LongRunningInfo("The LongRunningProcedure has completed.", 100))
	End Sub

  End Class


End Namespace
