﻿Imports System.Windows
Imports System.Windows.Controls
Imports DomainModel
Imports IdeaBlade.EntityModel

Namespace SilverlightApplication1

  Partial Public Class MainPage
	  Inherits UserControl

	Private _entityManager As NorthwindIBEntities
	Private _owToken As String

	Public Sub New()
	  InitializeComponent()
	  AddHandler Loaded, AddressOf MainPage_Loaded
	End Sub

	Private Sub MainPage_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  ' When connecting to a distributed BOS, it's always a good idea
	  ' to set up an error handler and issue an explicit Connect.
	  _entityManager = New NorthwindIBEntities(False)
	  AddHandler _entityManager.EntityServerError, Sub(o, args) MessageBox.Show(args.Exception.Message)
	  _entityManager.ConnectAsync(Sub(op)
		  If op.HasError Then
			  MessageBox.Show(op.Error.ToString())
		  Else
			  Me.btnSubscribeOW.IsEnabled = True
			  Me.btnSubscribeLRP.IsEnabled = Me.btnSubscribeOW.IsEnabled
		  End If
	  End Sub)
	End Sub

	Private Sub btnSubscribeLRP_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  SubscribeLRP()
	End Sub

	Private Sub btnSubscribeOW_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  SubscribeOW()
	End Sub

	Private Sub btnUnsubscribeOW_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  UnsubscribeOW()
	End Sub

	#Region "LongRunningProcedure"
	' Subscribe to a LongRunningProcedure, which provides periodic status notifications.
	' When complete, we automatically unsubscribe from it.

	Private Sub SubscribeLRP()

	  ' Register for the service.
	  _entityManager.RegisterCallback("DomainModel.LongRunningProcedure, DomainModel", "MultiStepService", AddressOf LongRunningProcedureNotification, "lrp") ' user token -  the callback -  Method name -  Type name

	  Me.btnSubscribeLRP.IsEnabled = False
	  MessageBox.Show("You are now subscribed to the LongRunningProcedure.")
	End Sub


	Private Sub LongRunningProcedureNotification(ByVal op As SubscriptionOperation)
	  If op.HasError Then
		MessageBox.Show(op.Error.Message)
		Me.btnSubscribeLRP.IsEnabled = True
		Return
	  End If

	  ' We know this service returns one arg.
	  Dim info As LongRunningInfo = TryCast(op.NotificationData(0), LongRunningInfo)
	  Dim msg As String = info.Message
	  Dim workComplete As Integer = info.CompletionPercent
	  MessageBox.Show(msg, "Received from LongRunningProcedure", MessageBoxButton.OK)

	  ' We can unsubscribe once the procedure completes.
	  If workComplete = 100 Then
		op.CancelCallback()
		Me.btnSubscribeLRP.IsEnabled = True
	  End If
	End Sub
	#End Region

	#Region "OrderWatcher subscribe/unsubscribe"

	' Remember to add orders (manually or through another program) to
	' have OrderWatcher broadcast messages.
	' This will do the job:
	'     insert into [Order](customerid, employeeid, orderdate, rowversion)
	'     select customerid, 1, getdate(), 1 from customer where CompanyName = 'QUICK-Stop'

	Private Sub SubscribeOW()
	  _owToken = "2" ' Arbitrary user token - we'll use this to unsubscribe.
	  Dim employeeId As Integer = 1 ' We'll pass this as a "client argument" into the service.

	  ' Register
	  _entityManager.RegisterCallback("DomainModel.OrderWatcher, DomainModel", "NewOrderService", AddressOf OrderPushCallback, _owToken, employeeId) ' client argument(s) -  user token -  the callback -  Method name -  Type name

	  ' This example contains only a single subscription to this service from this client.  You
	  ' can subscribe to this or other services multiple times - each subscription
	  ' needs to use a unique UserToken.
	  Me.btnUnsubscribeOW.IsEnabled = True
	  Me.btnSubscribeOW.IsEnabled = False

	  MessageBox.Show("You are now subscribed to the OrderWatcher.")
	End Sub

	Private Sub UnsubscribeOW()
	  ' Cancel based on the UserToken we subscribed with.
	  _entityManager.CancelCallback(_owToken)

	  Me.btnUnsubscribeOW.IsEnabled = False
	  Me.btnSubscribeOW.IsEnabled = True
	  MessageBox.Show("You are now unsubscribed from the OrderWatcher.")
	End Sub

	Private Sub OrderPushCallback(ByVal op As SubscriptionOperation)
	  If op.HasError Then
		MessageBox.Show(op.Error.Message)
		Me.btnUnsubscribeOW.IsEnabled = False
		Me.btnSubscribeOW.IsEnabled = True
		Return
	  End If
	  ' We know the service is returning a single message string...
	  Dim msg As String = op.NotificationData(0).ToString()
	  MessageBox.Show(msg, "Received from OrderWatcher", MessageBoxButton.OK)
	End Sub

	#End Region
  End Class
End Namespace
